# openclip -- a python library for working with
# openclip xml files
#
# @author: Robert Moggach <rob@moggach.com>
#
# openclip/core.py -- the core class
#
import os
import uuid
import glob
import re
from datetime import datetime
from copy import deepcopy
import util
from exceptions import *

from . import constants as _c
from . import LOG
from .elements import Clip

try:
    from lxml import etree as ET
    LOG.debug("running with lxml.etree")
except ImportError:
    try:
        # Python 2.5
        import xml.etree.cElementTree as ET
        LOG.warning("running with cElementTree on Python 2.5 (or higher)")
    except ImportError:
        try:
            import xml.etree.ElementTree as ET
            LOG.warning("running with ElementTree on Python 2.5  (or higher)")
        except ImportError:
            try:
                # normal cElementTree install
                import cElementTree as ET
                LOG.warning("running with cElementTree")
            except ImportError:
                try:
                    # normal ElementTree install
                    import elementtree.ElementTree as ET
                    LOG.warning("running with ElementTree")
                except ImportError:
                    LOG.critical("Failed to import ElementTree from any known place")


__all__ = [ 'OpenClip' ]


class OpenClip(object):
    PARSER = ET.XMLParser(remove_blank_text=True) if util.lxml_installed() else ET.XMLParser()
    '''
    this class simplifies the loading/writing of an OpenClip file
    '''
    childrenRequired = [ 'tracks', 'versions' ]
    childrenOptional = [ 'handler', 'name', 'userData', 'comment', 'sourceName', 'startTimeCode', 'duration', 'editRate', 'dropMode' ]
    def __init__(self):
        pass

    def __init__(self, src_xml_path=None):
        # set the src_xml_path from which we read the XML file
        self.src_xml_path = src_xml_path
        # set the basename of the file from the src_xml_path
        # this is a "cleaning" procedure
        # basename is used as a fallback for file name and clip name
        self.basename = util.get_file_basename(src_xml_path)
        if src_xml_path is None:
            self.dest_xml_path = util.get_cwd_or_tempdir()
            self.clip = Clip(self.src_element)
        else:
            # get the actual file path to write to
            # dest_xml_path has custom setter/getter so...
            # ** this may return a new path in the system tempdir **
            self.dest_xml_path = self.src_xml_path
            # send the harvested XML to the Clip object
            # which will then pass on to Tracks, Versions, Etc.
            self.clip = Clip( self.src_element )

    def __unicode__(self):
        # this returns the object etree element as a string
        # based on the class name (using tag here)
        return util.element_to_xml(self.etree)

    def __str__(self):
        # return the unicode string encoded properly
        return self.__unicode__().encode('utf8', 'replace')

    @property
    def etree(self):
        return ET.ElementTree(self.clip.element)

    @property
    def root(self):
        return self.etree.getroot()

    @property
    def xml(self):
        return util.element_to_xml(self.etree)

    def _set_dest_name(self, value):
        basename = get_file_basename(value)
        self._dest_name='{file_name}{ext}'.format(file_name=basename, ext=_c.OPENCLIP_EXT)

    def _get_dest_name(self):
        return '{file_name}{ext}'.format(file_name=self.basename, ext=_c.OPENCLIP_EXT)

    dest_name = property(_get_dest_name, _set_dest_name)

    def _set_dest_xml_path(self, value):
        file_path = value
        # # if file write status is None, file does not exist but path is writable
        if value is None or not util.clip_is_writable(value):
            LOG.warning('not a valid clip file path: {path}'.format(path=value))
            cwd = os.getcwd()
            file_path = os.path.join(cwd, self.dest_name)
            if not util.clip_is_writable(file_path):
                file_path = os.path.join(util.get_tempdir(), self.dest_name)
                LOG.info('new clip file destination set to: {path}'.format(path=file_path))
            else:
                LOG.info('new clip file destination set to: {path}'.format(path=file_path))
        self._dest_xml_path = file_path

    def _get_dest_xml_path(self):
        return self._dest_xml_path

    dest_xml_path = property(_get_dest_xml_path, _set_dest_xml_path, doc="destination path to write file to")

    def create_etree(self):
        root_xml_fmt = '<?xml version="1.0" encoding="utf-8"?><clip type="clip" version="3"><name type="string">{clip_name}</name></clip>'
        root_xml = root_xml_fmt.format(clip_name=self.basename)
        root = ET.XML(root_xml)
        etree = ET.ElementTree(root)
        return etree

    def _get_src_element(self):
        try:
            try:
                # parse the document and return a clip file?
                src_element = ET.parse(self.src_xml_path, parser=OpenClip.PARSER).getroot()
            except ET.XMLSyntaxError as e:
                # not an XML file...
                raise InvalidOpenClipFile, e
            except IOError as e:
                # not a file!
                raise InvalidOpenClipFile, e
            if not ET.iselement(src_element) and not src_element.tag=='clip':
                # XML but NOT a clip file
                raise InvalidOpenClipFile, 'Cannot retrieve root <clip> Element.'
        except InvalidOpenClipFile as e:
            # ok so we tried but we'll have to init a new etree
            LOG.debug(e)
            src_element = self.create_etree().getroot()
        except:
            LOG.debug('Unknown Error parsing file... Does it exist?')
            LOG.warning('could not parse file. Generating generic clip XML without track or version.')
            src_element = self.create_etree().getroot()
        return src_element

    def _set_src_element(self, element):
        if not ET.iselement(element) and not element.tag=='clip':
            self._src_element = self.create_etree().getroot()
            LOG.warning('rejected non-element or non-clip.')
        else:
            self._src_element = element

    src_element = property(_get_src_element, _set_src_element, doc="Source XML ElementTree/Document")

    def write_clip_file(self):
        with open(self.dest_xml_path, 'w') as outfile:
            outfile.write(self.xml)

