# openclip -- a python library for working with
# openclip xml files
#
# @author: Robert Moggach <rob@moggach.com>
#
# openclip/util.py -- utility classes and functions
#
import os
import shutil
import re
import uuid
import glob
import ntpath
import tempfile
import errno
from datetime import datetime
import tempfile

try:
    from lxml import etree as ET
    print("running with lxml.etree")
except ImportError:
    try:
        # Python 2.5
        import xml.etree.cElementTree as ET
        print("running with cElementTree on Python 2.5+")
    except ImportError:
        try:
            import xml.etree.ElementTree as ET
            print("running with ElementTree on Python 2.5+")
        except ImportError:
            try:
                # normal cElementTree install
                import cElementTree as ET
                print("running with cElementTree")
            except ImportError:
                try:
                    # normal ElementTree install
                    import elementtree.ElementTree as ET
                    print("running with ElementTree")
                except ImportError:
                    print("Failed to import ElementTree from any known place")


__all__ = [ 'color', 'lxml_installed', 'dir_is_writable', 'clip_is_writable', 'get_tempdir', 'can_write_path', 'get_file_basename', 'clean_version', 'get_frame_pad', 'element_to_xml', 'make_backup_file' ]


class color:
    """
    simple class for easier
    access to shell color escape codes
    Eg.
        print color.BOLD+"Bold Text"+color.END
    """
    PURPLE = '\033[95m'
    CYAN = '\033[96m'
    DARKCYAN = '\033[36m'
    BLUE = '\033[94m'
    GREEN = '\033[92m'
    YELLOW = '\033[93m'
    RED = '\033[91m'
    BOLD = '\033[1m'
    UNDERLINE = '\033[4m'
    END = '\033[0m'


def lxml_installed():
    import imp
    try:
        imp.find_module('lxml')
        return True
    except ImportError:
        return False


def dir_is_writable(path):
    if not os.path.exists(path):
        if os.path.splitext(path)[-1] in ['.clip', '.xml']:
            path = os.path.dirname(path)
    if os.path.isfile(path):
        path = os.path.dirname(path)
    if not os.path.isdir(path):
        return False
    try:
        testfile = tempfile.TemporaryFile(dir = path)
        testfile.close()
    except OSError as e:
        if e.errno == errno.EACCES:  # 13
            return False
        e.filename = path
        raise
    return True


def clip_is_writable(path):
    if not os.path.splitext(path)[-1] in ['.clip', '.xml']:
        # path is not a clip file
        return False
    else:
        if os.path.exists(path):
            if not os.path.isfile(path):
                # exists but not a file
                return False
            elif os.access(path, os.W_OK):
                # file exists and is writable
                return True
            else:
                # exists but not writable
                return False
        else:
            return dir_is_writable(path)


def get_tempdir():
    tempdir=os.path.join(tempfile.gettempdir(),'openclip')
    try:
        os.makedirs(tempdir)
        return tempdir
    except OSError:
        return None


def can_write_path(file_path):
    """
    simple function that returns:
       - True if file exists
       - None if path is writable and no file exists
       - False if file exists and is not writable
       - False if file path is not writable

    To use this effectively you need to check explicitly for True,False,or None
    """
    if os.path.isfile(file_path): # file exists
        return True
    elif os.access(os.path.dirname(file_path), os.W_OK): # file does not exist but path is writable
        return None
    else: # can not write thus stdout?
        return False


def get_file_basename(file_path):
    '''
    get the basename of a file_path
    Eg. '/Users/rob/test.clip' returns 'test'
    '''
    if not file_path is None:
        return os.path.splitext(os.path.basename(file_path))[0]
    else:
        return get_temp_name_str()


def get_cwd_or_tempdir():
    cwd = os.getcwd()
    if dir_is_writable(cwd):
        return cwd
    else:
        return get_tempdir


def get_temp_name_str():
    dt = datetime.now()
    datetime_str = '{:%Y%m%d_%H%M%S}'.format(dt)
    return 'OpenClip_{datetime_str}_{microsec}'.format(datetime_str=datetime_str,microsec=str(dt.microsecond)[:2])


def clean_version(version_str, version_format_str):
    """
    simple function that takes an
    input version string, sanitizes it,
    then reformats using a version format string
    similar to: "v{:0>3d}"
    Eg. V0003 becomes v003
    """
    try:
        version = int(''.join(i for i in version_str if i.isdigit()))
    except ValueError:
        version = 404
    return version_format_str.format(version)


def get_frame_pad(path, default_padding=4):
    """
    utility function to return padding of
    image sequence frame numbers Eg. 4
    """
    path = re.sub(r"[.]%[\d]+d[.]([\w]+)$", r".*.\1", path) # Ex: '/foo/bar/baz.%04d.exr' -> '/foo/bar/baz.*.exr'
    for filename in glob.iglob(path):
        return len(re.search(r"[.]([\d]+)[.][\w]+$", filename).group(1)) # Ex: '.0001.exr' -> groups: ('0001')
    return default_padding


def element_to_xml(etree_element, xml_header=True, encoding='utf-8', pretty_print=False):
    """
    simple function to return serialized element as unicode encoded pretty XML
    """
    try:
        return ET.tostring(etree_element, xml_declaration=xml_header, encoding=encoding, pretty_print=pretty_print)
    except TypeError:
        return None


def make_backup_file(file_path):
    filename = ntpath.basename(file_path)
    # Create a backup of the original file
    bakfile = "{file_path}.bak".format(file_path=file_path)
    if not os.path.isfile(bakfile):
        shutil.copy2(file_path, bakfile)
    else:
        created = False
        for i in range ( 1, 99 ):
            bakfile = "%s.bak.%02d" % ( file_path, i )
            if not os.path.isfile(bakfile):
                shutil.copy2(file_path,bakfile)
                created = True
                break
        if not created:
            bakfile = "%s.bak.last" % file_path
            shutil.copy2(file_path, bakfile)
